@@ -389,7 +389,15 @@ void PrintFormatNumber(void * ValuePtr, int Format, int ByteCount)
                 s = 8;
                 break;
 
-            case FMT_SINGLE:    printf("%f",(double)*(float *)ValuePtr); s=8; break;
+            case FMT_SINGLE:
+                {
+                    float f;
+                    int tmp = *(int*)ValuePtr;
+                    f = *(float *)&tmp;
+                    printf("%f",f);
+                }
+                s=4;
+                break;
             case FMT_DOUBLE:    printf("%f",*(double *)ValuePtr);        s=8; break;
             default: 
                 printf("Unknown format %d:", Format);
@@ -441,9 +449,20 @@ double ConvertAnyFormat(void * ValuePtr, int Format)
         case FMT_SSHORT:    Value = (signed short)Get16u(ValuePtr);  break;
         case FMT_SLONG:     Value = Get32s(ValuePtr);                break;
 
-        // Not sure if this is correct (never seen float used in Exif format)
-        case FMT_SINGLE:    Value = (double)*(float *)ValuePtr;      break;
-        case FMT_DOUBLE:    Value = *(double *)ValuePtr;             break;
+        // Never seen floats used in actual exif format,
+        // this code only ever hit with fuzz testing.
+        // This code may not necessaryly print correct values if float *were*
+        // to be used in exif, as it doesn't define which floating point
+        // stanard is to be used.
+        case FMT_SINGLE:
+            {
+                int tmp = *(int*)ValuePtr;
+                Value = *(float *)&tmp;
+            }
+            break;
+
+        case FMT_DOUBLE:
+            Value = *(double *)ValuePtr;             break;
 
         default:
             ErrNonfatal("Illegal format code %d in Exif header",Format,0);
@@ -463,6 +482,7 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
     int ThumbnailOffset = 0;
     int ThumbnailSize = 0;
     char IndentString[25];
+    unsigned OffsetVal;
 
     if (NestingLevel > 4){
         ErrNonfatal("Maximum Exif directory nesting exceeded (corrupt Exif header)", 0,0);
@@ -510,6 +530,12 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
         Format = Get16u(DirEntry+2);
         Components = Get32u(DirEntry+4);
 
+        if (Components > 0x10000){
+            //Components count too large could cause overflow on subsequent check
+            ErrNonfatal("Bad components count %x", Components,0);
+            continue;
+        }
+
         if ((Format-1) >= NUM_FORMATS) {
             // (-1) catches illegal zero case as unsigned underflows to positive large.
             ErrNonfatal("Illegal number format %d for tag %04x in Exif", Format, Tag);
@@ -524,17 +550,16 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
         ByteCount = Components * BytesPerFormat[Format];
 
         if (ByteCount > 4){
-            int OffsetVal;
             OffsetVal = Get32u(DirEntry+8);
             // If its bigger than 4 bytes, the dir entry contains an offset.
-            if (OffsetVal+ByteCount > ExifLength || OffsetVal < 0 || OffsetVal > 65536){
+            if (OffsetVal+ByteCount > (unsigned)ExifLength || OffsetVal > 65536){
                 // Bogus pointer offset and / or bytecount value
                 ErrNonfatal("Illegal value pointer for tag %04x in Exif", Tag,0);
                 continue;
             }
             ValuePtr = OffsetBase+OffsetVal;
 
-            if (OffsetVal > ImageInfo.LargestExifOffset){
+            if (OffsetVal > (unsigned)ImageInfo.LargestExifOffset){
                 ImageInfo.LargestExifOffset = OffsetVal;
             }
 
@@ -571,7 +596,13 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
             switch(Format){
                 case FMT_BYTE:
                     if(ByteCount>1){
-                        printf("%.*ls\n", ByteCount/2, (wchar_t *)ValuePtr);
+                        for (a=0;a<ByteCount;a+=2){
+                            int cv = *(char *)(ValuePtr+a)+(*(char *)(ValuePtr+a+1)<<8);
+                            // Note that after getting the 16-bit char, putchar truncates it back
+                            // down to 8 bit.  Unicoe and linus console is something I don't understand.
+                            putchar(cv);
+                        }
+                        putchar('\n');
                     }else{
                         PrintFormatNumber(ValuePtr, Format, ByteCount);
                         printf("\n");
@@ -622,15 +653,16 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                 break;
 
             case TAG_DATETIME_ORIGINAL:
-                // If we get a DATETIME_ORIGINAL, we use that one.
-                strncpy(ImageInfo.DateTime, (char *)ValuePtr, 19);
-                // Fallthru...
-
             case TAG_DATETIME_DIGITIZED:
             case TAG_DATETIME:
-                if (!isdigit(ImageInfo.DateTime[0])){
+                if (ValuePtr+19 >= OffsetBase+ExifLength){
+                    ErrNonfatal("Incomplete time",0,0);
+                    continue;
+                }
+                
+                if (Tag == TAG_DATETIME_ORIGINAL || !isdigit(ImageInfo.DateTime[0])){
                     // If we don't already have a DATETIME_ORIGINAL, use whatever
-                    // time fields we may have.
+                    // time fields we may have.  But if ORIGINAL tag comes later, use that one.
                     strncpy(ImageInfo.DateTime, (char *)ValuePtr, 19);
                 }
 
@@ -642,20 +674,6 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                     (char *)ValuePtr - (char *)OffsetBase;
                 break;
 
-            case TAG_WINXP_COMMENT:
-                if (ImageInfo.Comments[0]){ // We already have a jpeg comment.
-                    // Already have a comment (probably windows comment), skip this one.
-                    if (ShowTags) printf("Windows XP commend and other comment in header\n");
-                    break; // Already have a windows comment, skip this one.
-                }
-
-                if (ByteCount > 1){
-                    if (ByteCount > MAX_COMMENT_SIZE) ByteCount = MAX_COMMENT_SIZE;
-                    memcpy(ImageInfo.Comments, ValuePtr, ByteCount);
-                    ImageInfo.CommentWidthchars = ByteCount/2;
-                }
-                break;
-
             case TAG_USERCOMMENT:
                 if (ImageInfo.Comments[0]){ // We already have a jpeg comment.
                     // Already have a comment (probably windows comment), skip this one.
@@ -993,6 +1011,7 @@ void process_EXIF (unsigned char * ExifSection, int length)
     
     Clear_EXIF();
 
+
     if (ShowTags){
         printf("Exif header %u bytes long\n",length);
     }
@@ -1544,25 +1563,21 @@ void ShowImageInfo(int ShowFileInfo)
     if (ImageInfo.Comments[0]){
         int a,c;
         printf("Comment      : ");
-        if (!ImageInfo.CommentWidthchars){
-            for (a=0;a<MAX_COMMENT_SIZE;a++){
-                c = ImageInfo.Comments[a];
-                if (c == '\0') break;
-                if (c == '\n'){
-                    // Do not start a new line if the string ends with a carriage return.
-                    if (ImageInfo.Comments[a+1] != '\0'){
-                        printf("\nComment      : ");
-                    }else{
-                        printf("\n");
-                    }
+        for (a=0;a<MAX_COMMENT_SIZE;a++){
+            c = ImageInfo.Comments[a];
+            if (c == '\0') break;
+            if (c == '\n'){
+                // Do not start a new line if the string ends with a carriage return.
+                if (ImageInfo.Comments[a+1] != '\0'){
+                    printf("\nComment      : ");
                 }else{
-                    putchar(c);
+                    printf("\n");
                 }
+            }else{
+                putchar(c);
             }
-            printf("\n");
-        }else{
-            printf("%.*ls\n", ImageInfo.CommentWidthchars, (wchar_t *)ImageInfo.Comments);
         }
+        printf("\n");
     }
 }
 
