@@ -88,6 +88,12 @@ void ProcessGpsInfo(unsigned char * DirStart, unsigned char * OffsetBase, unsign
         Format = Get16u(DirEntry+2);
         Components = Get32u(DirEntry+4);
 
+        if (Components > 0x10000){
+            //Components count too large could cause overflow on subsequent check
+            ErrNonfatal("Bad components count %x", Components,0);
+            continue;
+        }
+
         if ((Format-1) >= NUM_FORMATS) {
             // (-1) catches illegal zero case as unsigned underflows to positive large.
             ErrNonfatal("Illegal number format %d for Exif gps tag %04x", Format, Tag);
@@ -101,7 +107,7 @@ void ProcessGpsInfo(unsigned char * DirStart, unsigned char * OffsetBase, unsign
             unsigned OffsetVal;
             OffsetVal = Get32u(DirEntry+8);
             // If its bigger than 4 bytes, the dir entry contains an offset.
-            if (OffsetVal > 0x1000000 || OffsetVal+ByteCount > ExifLength){
+            if (OffsetVal+ByteCount > ExifLength || OffsetVal > 65536){
                 // Max exif in jpeg is 64k, so any offset bigger than that is bogus.
                 // Bogus pointer offset and / or bytecount value
                 ErrNonfatal("Illegal value pointer for Exif gps tag %04x", Tag,0);
@@ -148,6 +154,7 @@ void ProcessGpsInfo(unsigned char * DirStart, unsigned char * OffsetBase, unsign
                     Values[a] = ConvertAnyFormat(ValuePtr+a*ComponentSize, Format);
                 }
 
+
                 snprintf(TempString, sizeof(TempString), FmtString, Values[0], Values[1], Values[2]);
 
                 if (Tag == TAG_GPS_LAT){
