@@ -2,14 +2,14 @@
 // Program to pull the information out of various types of EXIF digital 
 // camera files and show it in a reasonably consistent way
 //
-// Version 3.03
+// Version 3.06
 //
 // Compiling under Windows:  
 //   Make sure you have Microsoft's compiler on the path, then run make.bat
 //
-// Dec 1999 - Dec 2018
+// Dec 1999 - Oct 2020
 //
-// by Matthias Wandel   www.sentex.net/~mwandel
+// by Matthias Wandel   http://woodgears.ca
 //--------------------------------------------------------------------------
 #ifdef _WIN32
     #include <io.h>
@@ -19,11 +19,11 @@
 
 #include <sys/stat.h>
 
-#define JHEAD_VERSION "3.03"
+#define JHEAD_VERSION "3.06"
 
 // This #define turns on features that are too very specific to 
 // how I organize my photos.  Best to ignore everything inside #ifdef MATTHIAS
-//#define MATTHIAS
+#define MATTHIAS
 
 
 // Bitmasks for DoModify:
@@ -60,8 +60,9 @@ static int TrimExifTrailingZeroes = FALSE;
 static char * ApplyCommand = NULL;  // Apply this command to all images.
 static char * FilterModel = NULL;
 static int    FilterQuality = 0;
-static int    ExifOnly    = FALSE;
-static int    PortraitOnly = FALSE;
+static int    ExifOnly    = FALSE;  // Only do images with exif header
+static int    ProcessOnly = -1;     // 0 for baseline, 2 for progressive only, -1 for all images.
+static int    PortraitOnly = FALSE; // Only do images with portrait orientation.
 static time_t ExifTimeAdjust = 0;   // Timezone adjust
 static time_t ExifTimeSet = 0;      // Set exif time to a value.
 static char DateSet[11];
@@ -85,7 +86,7 @@ static char * ExifXferScrFile = NULL;// Extract Exif header from this file, and
                                     // put it into the Jpegs processed.
 
 static int EditComment = FALSE;     // Invoke an editor for editing the comment
-static int SupressNonFatalErrors = FALSE; // Wether or not to pint warnings on recoverable errors
+static int SuppressNonFatalErrors = FALSE; // Wether or not to pint warnings on recoverable errors
 
 static char * CommentSavefileName = NULL; // Save comment to this file.
 static char * CommentInsertfileName = NULL; // Insert comment from this file.
@@ -124,7 +125,7 @@ void ErrFatal(const char * msg)
 //--------------------------------------------------------------------------
 void ErrNonfatal(const char * msg, int a1, int a2)
 {
-    if (SupressNonFatalErrors) return;
+    if (SuppressNonFatalErrors) return;
 
     fprintf(stderr,"\nNonfatal Error : ");
     if (CurrentFile) fprintf(stderr,"'%s' ",CurrentFile);
@@ -448,6 +449,11 @@ static int CheckFileSkip(void)
 {
     // I sometimes add code here to only process images based on certain
     // criteria - for example, only to convert non progressive Jpegs to progressives, etc..
+    if(ProcessOnly >= 0 && (ImageInfo.Process & 0x0f) != ProcessOnly){
+        // ProcessOnly == 0 means skip baseline oencoded jpegs
+        // ProcessOnly == 2 means skip progressive oencoded jpegs
+        return TRUE;
+    }
 
     if (FilterModel){
         // Filtering processing by camera model.
@@ -584,6 +590,9 @@ static void DoFileRenaming(const char * FileName)
 
         if (FileName[a] == '.') ExtensionPart = a;  // Remember where extension starts.
     }
+    if (ExtensionPart < PrefixPart) { // no extension found
+        ExtensionPart = strlen(FileName);
+    } 
 
     if (!Exif2tm(&tm, ImageInfo.DateTime)){
         printf("File '%s' contains no exif date stamp.  Using file date\n",FileName);
@@ -657,7 +666,7 @@ static void DoFileRenaming(const char * FileName)
 
     AddLetter = isdigit(NewBaseName[strlen(NewBaseName)-1]);
     for (a=0;;a++){
-        char NewName[PATH_MAX+10];
+        char NewName[PATH_MAX*2+10];
         char NameExtra[3];
         struct stat dummy;
 
@@ -677,7 +686,7 @@ static void DoFileRenaming(const char * FileName)
             NameExtra[0] = 0;
         }
 
-        sprintf(NewName, "%s%s.jpg", NewBaseName, NameExtra);
+        snprintf(NewName, sizeof(NewName), "%s%s.jpg", NewBaseName, NameExtra);
 
         if (!strcmp(FileName, NewName)) break; // Skip if its already this name.
 
@@ -773,7 +782,7 @@ static int RegenerateThumbnail(const char * FileName)
         return FALSE;
     }
 
-    sprintf(ThumbnailGenCommand, "mogrify -thumbnail %dx%d \"%s\"", 
+    sprintf(ThumbnailGenCommand, "mogrify -thumbnail %dx%d -quality 80 \"%s\"", 
         RegenThumbnail, RegenThumbnail, FileName);
 
     if (system(ThumbnailGenCommand) == 0){
@@ -1259,7 +1268,7 @@ badtime:
 static void Usage (void)
 {
     printf("Jhead is a program for manipulating settings and thumbnails in Exif jpeg headers\n"
-           "used by most Digital Cameras.  v"JHEAD_VERSION" Matthias Wandel, Dec 31 2018.\n"
+           "used by most Digital Cameras.  v"JHEAD_VERSION" Matthias Wandel, Oct 5 2020.\n"
            "http://www.sentex.net/~mwandel/jhead\n"
            "\n");
 
@@ -1274,7 +1283,7 @@ static void Usage (void)
            "  -dc        Delete comment field (as left by progs like Photoshop & Compupic)\n"
            "  -de        Strip Exif section (smaller JPEG file, but lose digicam info)\n"
            "  -di        Delete IPTC section (from Photoshop, or Picasa)\n"
-           "  -dx        Deletex XMP section\n"
+           "  -dx        Delete XMP section\n"
            "  -du        Delete non image sections except for Exif and comment sections\n"
            "  -purejpg   Strip all unnecessary data from jpeg (combines -dc -de and -du)\n"
            "  -mkexif    Create new minimal exif section (overwrites pre-existing exif)\n"
@@ -1304,8 +1313,7 @@ static void Usage (void)
            "             '%%i' will include a sequence number, starting from 1. You can\n"
            "             You can specify '%%03i' for example to get leading zeros.\n"
            "             This feature is useful for ordering files from multiple digicams to\n"
-           "             sequence of taking.  Only renames files whose names are mostly\n"
-           "             numerical (as assigned by digicam)\n"
+           "             sequence of taking.\n"
            "             The '.jpg' is automatically added to the end of the name.  If the\n"
            "             destination name already exists, a letter or digit is added to \n"
            "             the end of the name to make it unique.\n"
@@ -1364,7 +1372,7 @@ static void Usage (void)
            "  -q         Quiet (no messages on success, like Unix)\n"
            "  -V         Show jhead version\n"
            "  -exifmap   Dump header bytes, annotate.  Pipe thru sort for better viewing\n"
-           "  -se        Supress error messages relating to corrupt exif header structure\n"
+           "  -se        Suppress error messages relating to corrupt exif header structure\n"
            "  -c         concise output\n"
            "  -nofinfo   Don't show file info (name/size/date)\n"
 
@@ -1505,19 +1513,19 @@ int main (int argc, char **argv)
             CreateExifSection = TRUE;
             DoModify |= MODIFY_JPEG;
     // Output verbosity control
-        }else if (!strcmp(arg,"-h")){
+        }else if (!strcmp(arg,"-h") || !strcmp(arg,"--help")){
             Usage();
         }else if (!strcmp(arg,"-v")){
             ShowTags = TRUE;
         }else if (!strcmp(arg,"-q")){
             Quiet = TRUE;
         }else if (!strcmp(arg,"-V")){
-            printf("Jhead version: "JHEAD_VERSION"   Compiled: "__DATE__"\n");
+            printf("Jhead version: "JHEAD_VERSION"\n");
             exit(0);
         }else if (!strcmp(arg,"-exifmap")){
             DumpExifMap = TRUE;
         }else if (!strcmp(arg,"-se")){
-            SupressNonFatalErrors = TRUE;
+            SuppressNonFatalErrors = TRUE;
         }else if (!strcmp(arg,"-c")){
             ShowConcise = TRUE;
         }else if (!strcmp(arg,"-nofinfo")){
@@ -1601,7 +1609,7 @@ int main (int argc, char **argv)
             if (pOldDate){
                 OldDate = ParseCmdDate(pOldDate+1);
             }else{
-                ErrFatal("Must specifiy second date for -da option");
+                ErrFatal("Must specify second date for -da option");
             }
             if (ExifTimeAdjust) ErrFatal("Can only use one of -da or -ta options at once");
             ExifTimeAdjust = NewDate-OldDate;
@@ -1672,6 +1680,11 @@ int main (int argc, char **argv)
 			if (sscanf(argv[++argn], "%d", &FilterQuality) != 1){
 				Usage();
 			}
+        }else if (!strcmp(arg,"-proc")){
+            sscanf(argv[++argn], "%d", &ProcessOnly);
+            if (ProcessOnly < 0 || ProcessOnly > 2){
+                ErrFatal("-proc must be followed by a number 0-2");
+            }
         }else if (!strcmp(arg,"-exonly")){
             ExifOnly = 1;
         }else if (!strcmp(arg,"-orp")){
@@ -1715,7 +1728,7 @@ int main (int argc, char **argv)
 
     if (ThumbSaveName != NULL && strcmp(ThumbSaveName, "&i") == 0){
         printf("Error: By specifying \"&i\" for the thumbail name, your original file\n"
-               "       will be overwitten.  If this is what you really want,\n"
+               "       will be overwritten.  If this is what you really want,\n"
                "       specify  -st \"./&i\"  to override this check\n");
         exit(0);
     }
