@@ -48,7 +48,7 @@ static void process_COM (const uchar * Data, int length)
     for (a=2;a<length;a++){
         ch = Data[a];
 
-        if (ch == '\r' && Data[a+1] == '\n') continue; // Remove cr followed by lf.
+        if (ch == '\r' && a < length-1 && Data[a+1] == '\n') continue; // Remove cr followed by lf.
 
         if (ch >= 32 || ch == '\n' || ch == '\t'){
             Comment[nch++] = (char)ch;
@@ -64,7 +64,6 @@ static void process_COM (const uchar * Data, int length)
     }
 
     strcpy(ImageInfo.Comments,Comment);
-    ImageInfo.CommentWidthchars = 0;
 }
 
  
@@ -170,7 +169,10 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
 
         Sections[SectionsRead].Size = itemlen;
 
-        Data = (uchar *)malloc(itemlen);
+        // Allocate an extra 20 bytes more than needed, because sometimes when reading structures,
+        // if the section erroneously ends before short structures that should be there, that can trip
+        // memory checkers in combination with fuzzers.
+        Data = (uchar *)malloc(itemlen+20);
         if (Data == NULL){
             ErrFatal("Could not allocate memory");
         }
@@ -248,13 +250,13 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                 // marker instead, althogh ACDsee will write images with both markers.
                 // this program will re-create this marker on absence of exif marker.
                 // hence no need to keep the copy from the file.
-                if (memcmp(Data+2, "JFIF\0",5)){
-                    fprintf(stderr,"Header missing JFIF marker\n");
-                }
                 if (itemlen < 16){
                     fprintf(stderr,"Jfif header too short\n");
                     goto ignore;
                 }
+                if (memcmp(Data+2, "JFIF\0",5)){
+                    fprintf(stderr,"Header missing JFIF marker\n");
+                }
 
                 ImageInfo.JfifHeader.Present = TRUE;
                 ImageInfo.JfifHeader.ResolutionUnits = Data[9];
@@ -326,6 +328,10 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
             case M_SOF13:
             case M_SOF14:
             case M_SOF15:
+                if (itemlen < 8){
+                    fprintf(stderr,"Section too short\n");
+                    break;
+                }
                 process_SOFn(Data, marker);
                 break;
             default:
@@ -474,6 +480,8 @@ int ReplaceThumbnail(const char * ThumbFileName)
         }
 
         ThumbLen = 0;
+
+
         ThumbnailFile = NULL;
     }
 
@@ -653,12 +661,13 @@ int RemoveSectionType(int SectionType)
 {
     int a;
     int retval = FALSE;
+
     for (a=0;a<SectionsRead-1;a++){
         if (Sections[a].Type == SectionType){
             // Free up this section
             free (Sections[a].Data);
             // Move succeding sections back by one to close space in array.
-            memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a));
+            memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a-1));
             SectionsRead -= 1;
             a -= 1;
             retval = TRUE;
@@ -707,7 +716,7 @@ int RemoveUnknownSections(void)
                 // Unknown.  Delete.
                 free (Sections[a].Data);
                 // Move succeding sections back by one to close space in array.
-                memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a));
+                memmove(Sections+a, Sections+a+1, sizeof(Section_t) * (SectionsRead-a-1));
                 SectionsRead -= 1;
                 Modified = TRUE;
         }
